package org.memoriesdreamsandreflections.waipt_procedures.testutils ;


import java.io.File ;
import java.io.IOException ;
import java.util.ArrayList ;
import java.util.List ;

import org.jboss.arquillian.container.test.api.Testable ;
import org.jboss.shrinkwrap.api.ShrinkWrap ;
import org.jboss.shrinkwrap.api.exporter.ZipExporter ;
import org.jboss.shrinkwrap.api.spec.EnterpriseArchive ;
import org.jboss.shrinkwrap.api.spec.JavaArchive ;
import org.jboss.shrinkwrap.api.spec.WebArchive ;
import org.jboss.shrinkwrap.resolver.api.DependencyResolvers ;
import org.jboss.shrinkwrap.resolver.api.maven.EffectivePomMavenDependencyResolver ;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependency ;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependencyResolver ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.CombinedFilter ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.DependencyFilter ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.ExclusionFilter ;
import org.jboss.shrinkwrap.resolver.api.maven.filter.ScopeFilter ;
import org.memoriesdreamsandreflections.waipt_common.Globals ;
import org.memoriesdreamsandreflections.waipt_common.testutils.IntegrationTestUtil ;
import org.memoriesdreamsandreflections.waipt_common.testutils.WaiptConfigUtil ;
import org.memoriesdreamsandreflections.waipt_common.utils.PathUtil ;
import org.testng.Assert ;

public class ArchiveUtil
{

	/**
	 * 
	 * Builds the EAR once per test thread, and incorporates PID into ear name. Saves considerable time in integration test runs.
	 * @return
	 */
	public static EnterpriseArchive createTestArchive()
	{
		EnterpriseArchive l_ea = null ;
		
		/*
		 * 1. Determine where on file system we'll store EAR
		 */
		String l_tempDir = System.getProperty(  Globals.INTEGRATION_TEMP_DIR, "/tmp" ) ;

		/*
		 * 2. Build EAR File object
		 */
		String l_earTemplate = l_tempDir + "/test-procedures-" ; 
		File l_earFile = IntegrationTestUtil.produceIntegrationTestEarFile( l_earTemplate );
		
		/*
		 * 3. If EAR for this process already exists as a file, load it from the file system and return it
		 * 		to Arquillian
		 */
		if ( l_earFile.exists() )
		{
			l_ea = ShrinkWrap.createFromZipFile( EnterpriseArchive.class, l_earFile ) ;
			return l_ea ;
		}
		
		/*
		 * 5. During Maven executions, user.dir is project home directory i.e. base path for all artifacts 
		 */
		PathUtil l_path = new PathUtil( System.getProperty( "user.dir" ) ) ;
		
		/*
		 * 6. Prepare all Maven dependencies for 'waipt-ejb-procedures'
		 */
		EffectivePomMavenDependencyResolver resolver = DependencyResolvers.use( MavenDependencyResolver.class )
			.loadEffectivePom( l_path.getPomXml() )
			.importAllDependencies() ;
		
		/*
		 * 7. Build EAR content
		 */
		
		/*
		 * 7.1: EAR archive itself
		 */
		l_ea = ShrinkWrap.create(  EnterpriseArchive.class, l_earFile.getName() ) ;

		/*
		 * 7.2 Pull filtered dependencies into EAR lib (see note below
		 */
		l_ea.addAsLibraries( dependencies_( resolver ) ) ;

		/*
		 * 7.3 Include test persistence unit as separate jar
		 */
		l_ea.addAsLibrary( persistenceXml_() )  ;
		
		/*
		 * 7.4 Include waipt-jar-entities (Persistence Unit's Entity beans)
		 */
		l_ea.addAsLibrary( entities_( resolver ) ) ;
		
		/*
		 * 7.5 Include waipt-jar-are (Adaptive rules engine)
		 */
		l_ea.addAsLibrary( are_( resolver ) ) ;

		/*
		 * 7.6 Integrated data source
		 */
		l_ea.addAsManifestResource( "waimap-ds.xml" ) ;

		/*
		 * 7.7 Add testable WAR
		 */
		WebArchive l_wa = Testable.archiveToTest( 
			ShrinkWrap.create( WebArchive.class, "test.war" )
				.addPackages( true, "org.memoriesdreamsandreflections.waipt_procedures" )
				.addAsWebInfResource( "beans.xml", "beans.xml" ) ) ;
		l_ea.addAsModule( l_wa ) ;
		
		/*
		 * 7.8 Add Lockbox Webservice App as nontestable WAR
		 */
		l_ea.addAsModule( lockbox_() );
		
		/*
		 * 7.9 Add JBoss deployment structure ( Bouncy Castle dependency, etc )
		 */
		l_ea.addAsManifestResource( "jboss-deployment-structure.xml" );
		
		/*
		 * 7.10 Write it to file system for next test
		 */
		try
		{
			l_ea.as(  ZipExporter.class ).exportTo( l_earFile, true );
			System.out.println( "@@@@TEST ARCHIVE WRITTEN: " + l_earFile.getCanonicalPath() );
		}
		catch(Exception e)
		{
			e.printStackTrace() ;
		}
		
		/*
		 * 7.11 Write application configuration information
		 */
		try
		{
			WaiptConfigUtil.createWaiptConfig( l_tempDir ) ;
			WaiptConfigUtil.createLockboxConfig( l_tempDir ) ;
		}
		catch( IOException e)
		{
			Assert.fail("Exception", e);
		}
		
		/*
		 * Return the archive to Arquillian
		 */
		return l_ea ;

	}
	
	static WebArchive lockbox_()
	{
		File l_lockboxWarFile = null ;
		try
		{
			l_lockboxWarFile = IntegrationTestUtil.produceArchiveFileFromMavenCoords( "org.memoriesdreamsandreflections:waipt-lockbox-war:war:0.1" );
			
		}
		catch(Exception e)
		{
			Assert.fail( "Exception " + e, e );
		}
		WebArchive l_lockboxWar = ShrinkWrap.createFromZipFile( WebArchive.class, l_lockboxWarFile ) ;
		l_lockboxWar.delete( "WEB-INF/lib");
		return l_lockboxWar ;
	}

	static File[] dependencies_( EffectivePomMavenDependencyResolver resolver )
	{
		/*
		 * 
		 * Shrinkwrap seems to run into some trouble when integration tests are run from the 'waipt-jee' level; 
		 * In that context, 'waipt-ejb-entities' and 'waipt-jar-are' were each being written to the EAR lib 
		 * twice: once with a normal version and once with a timestamp. This causes Integration testing to fail. 
		 * It's most likely caused by the way I'm using "resolver.resolveAsFiles()" but I'm not certain.
		 * Anyway I got things working by excluding the problematic dependencies from the resolver and then
		 * explicitly adding those same dependencies subsequently using 'addAsLibrary'. 
		 */
		CombinedFilter l_cf = new CombinedFilter(
			new ScopeFilter( "compile", "runtime", "test" ),
			new ExclusionFilter( "org.memoriesdreamsandreflections:waipt-ejb-entities:0.1" ),
			new ExclusionFilter( "org.memoriesdreamsandreflections:waipt-jar-are:0.1" ),
			new ExclusionFilter( "org.memoriesdreamsandreflections:waipt-lockbox-war:war:0.1" ) //,
					//new ArqShrinkExclusionFilter() 
		) ;
		return resolver.resolveAsFiles( l_cf ) ;
	}
	
	static JavaArchive persistenceXml_()
	{
		JavaArchive l_persistenceXml = ShrinkWrap.create( JavaArchive.class, "waipt-ee-persistence.jar" );
		l_persistenceXml.addAsManifestResource("META-INF/test-persistence.xml", "persistence.xml") ;
		return l_persistenceXml ;
	}
	
	static JavaArchive entities_( EffectivePomMavenDependencyResolver resolver )
	{
		List<JavaArchive> l_jaList = new ArrayList<JavaArchive>( resolver.resolveAs( JavaArchive.class, new DependencyFilter( "org.memoriesdreamsandreflections:waipt-ejb-entities:0.1" ) ) ) ;
		return l_jaList.get( 0 ) ;
	}
	
	static JavaArchive are_( EffectivePomMavenDependencyResolver resolver )
	{
		List<JavaArchive> l_jaList = new ArrayList<JavaArchive>( resolver.resolveAs( JavaArchive.class, new DependencyFilter( "org.memoriesdreamsandreflections:waipt-jar-are:0.1" ) ) ) ;
		return l_jaList.get( 0 ) ;
	}
	
	static class ArqShrinkExclusionFilter
	extends ExclusionFilter
	{
		public ArqShrinkExclusionFilter()
		{
			super( "" ) ;
		}

		@Override
		public boolean accept( MavenDependency element )
		{
			return element.coordinates().indexOf( "org.jboss.shrinkwrap" ) == -1
			&& element.coordinates().indexOf( "org.jboss.arquillian" ) == -1
			&& element.coordinates().indexOf( "testng" ) == -1 ;
		}
	}
	
}
